@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -22,6 +22,7 @@
 #define __BLUETOOTH_H__
 
 #include <drivers/atmodem/atutil.h>
+#include <ofono/dbus.h>
 
 /* AG supported features bitmap. Bluetooth HFP 1.5 spec page 77 */
 #define AG_FEATURE_3WAY 0x1
@@ -42,6 +43,14 @@
 #define HF_FEATURE_ENHANCED_CALL_STATUS 0x20
 #define HF_FEATURE_ENHANCED_CALL_CONTROL 0x40
 
+#define AG_CHLD_0 0x01
+#define AG_CHLD_1 0x02
+#define AG_CHLD_1x 0x04
+#define AG_CHLD_2 0x08
+#define AG_CHLD_2x 0x10
+#define AG_CHLD_3 0x20
+#define AG_CHLD_4 0x40
+
 enum hfp_indicator {
 	HFP_INDICATOR_SERVICE = 0,
 	HFP_INDICATOR_CALL,
@@ -55,10 +64,14 @@ enum hfp_indicator {
 
 struct hfp_data {
 	GAtChat *chat;
+	char *handsfree_path;
+	DBusMessage *slc_msg;
 	unsigned int ag_features;
+	unsigned int ag_mpty_features;
 	unsigned int hf_features;
 	unsigned char cind_pos[HFP_INDICATOR_LAST];
 	unsigned int cind_val[HFP_INDICATOR_LAST];
+	unsigned int at_timeout;
 };
 
 extern void hfp_netreg_init();
