@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -22,10 +22,14 @@
 enum sim_fileid {
 	SIM_EFPL_FILEID = 0x2f05,
 	SIM_EFLI_FILEID = 0x6f05,
+	SIM_EF_CPHS_MWIS_FILEID = 0x6f11,
+	SIM_EF_CPHS_INFORMATION_FILEID = 0x6f16,
+	SIM_EF_CPHS_MBDN_FILEID = 0x6f17,
 	SIM_EFMSISDN_FILEID = 0x6f40,
 	SIM_EFSPN_FILEID = 0x6f46,
 	SIM_EFSDN_FILEID = 0x6f49,
 	SIM_EFAD_FILEID = 0x6fad,
+	SIM_EFPHASE_FILEID = 0x6fae,
 	SIM_EFPNN_FILEID = 0x6fc5,
 	SIM_EFOPL_FILEID = 0x6fc6,
 	SIM_EFMBDN_FILEID = 0x6fc7,
@@ -59,6 +63,17 @@ struct sim_eons_operator_info {
 	char *info;
 };
 
+struct sim_ef_info {
+	unsigned short id;
+	unsigned short parent;
+	unsigned char file_type;
+	unsigned char size;
+	enum sim_file_access perm_read;
+	enum sim_file_access perm_update;
+};
+
+#define ROOTMF 0x3F00
+
 struct sim_eons *sim_eons_new(int pnn_records);
 void sim_eons_add_pnn_record(struct sim_eons *eons, int record,
 				const guint8 *tlv, int length);
@@ -93,3 +108,14 @@ gboolean sim_adn_parse(const unsigned char *data, int length,
 void sim_adn_build(unsigned char *data, int length,
 			const struct ofono_phone_number *ph,
 			const char *identifier);
+
+struct sim_ef_info *sim_ef_db_lookup(unsigned short efid);
+
+gboolean sim_parse_3g_get_response(const unsigned char *data, int len,
+					int *file_len, int *record_len,
+					int *structure, unsigned char *access,
+					unsigned short *efid);
+
+gboolean sim_parse_2g_get_response(const unsigned char *response, int len,
+					int *file_len, int *record_len,
+					int *structure, unsigned char *access);
